﻿using Microsoft.AspNetCore.Cors;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.IdentityModel.Tokens;

using MyBlog.Entity;
using MyBlog.IService;
using MyBlog.Model;
using MyBlog.Model.JWT;
using MyBlog.Shared.Extensions;
using MyBlog.Shared.Utility.ApiResult;
using MyBlog.Shared.Utility.MD5Utility;

using System;
using System.Collections.Generic;
using System.IdentityModel.Tokens.Jwt;
using System.Linq;
using System.Security.Claims;
using System.Text;
using System.Threading.Tasks;

namespace MyBlog.Jwt.Controllers
{
    [Route("api/[controller]/[action]")]
    [ApiController]
    public class AuthorizeController : ControllerBase
    {
        private readonly IWriterInfoService _writerInfoService;
        private readonly MyContext _myContext;
        private readonly TokenValidationParameters _tokenValidationParameters;

        public AuthorizeController(IWriterInfoService writerInfoService, MyContext myContext, TokenValidationParameters tokenValidationParameters)
        {
            _writerInfoService = writerInfoService;
            _myContext = myContext;
            _tokenValidationParameters = tokenValidationParameters;
        }

        [HttpPost]
        [DisableCors]
        public async Task<ApiResult> Login(string userName, string password)
        {
            string pwd = MD5Helper.MD5Encrypt32(password);
            var entity = await _writerInfoService.FindAsync(x => x.UserName == userName && x.UserPwd == pwd);

            if (entity != null)
            {
                var jwtToken = await GenerateJwtToken(entity);
                return new ApiResult(ResultCode.SUCCESS, "", jwtToken);
            }

            return new ApiResult(ResultCode.FALL, "Incorrect username or password");
        }

        [HttpPost]
        public async Task<ApiResult> RefreshToken(string token, string refreshToken)
        {
            if (ModelState.IsValid)
            {
                var result = await VerifyAndGenerateToken(token, refreshToken);

                return result == null ? new ApiResult(ResultCode.FALL) : new ApiResult(ResultCode.SUCCESS, result);
            }

            return new ApiResult(ResultCode.FALL);
        }

        private async Task<AuthResult> GenerateJwtToken(WriterInfo entity)
        {
            var jwtTokenHandler = new JwtSecurityTokenHandler();

            var claims = new Claim[]
                {
                    new Claim(ClaimTypes.Name,entity.Name),
                    new Claim("Id",entity.Id.ToString()),
                    new Claim("UserName",entity.UserName),
                    new Claim(JwtRegisteredClaimNames.Jti,Guid.NewGuid().ToString())
                };

            var key = new SymmetricSecurityKey(Encoding.UTF8.GetBytes("SDMC-CJAS1-SAD-DFSFA-SADHJVF-VF"));

            var isecurityToken = new JwtSecurityToken(
                issuer: "http://localhost:6060",
                audience: "http://localhost:5000",
                claims: claims,
                notBefore: DateTime.Now,
                expires: DateTime.Now.AddMinutes(1),
                signingCredentials: new SigningCredentials(key, SecurityAlgorithms.HmacSha256)
                );


            var token = jwtTokenHandler.WriteToken(isecurityToken);

            var refreshToken = new RefreshToken()
            {
                JwtId = isecurityToken.Id,
                IsUsed = false,
                IsRevorked = false,
                UserId = entity.Id,
                AddedDate = DateTime.UtcNow,
                ExpiryDate = DateTime.UtcNow.AddMinutes(5),
                Token = RandomString(35) + Guid.NewGuid(),
            };

            await _myContext.RefreshTokens.AddAsync(refreshToken);
            await _myContext.SaveChangesAsync();

            return new AuthResult
            {
                Token = token,
                RefreshToken = refreshToken.Token,
                Success = true,
            };
        }

        private async Task<AuthResult> VerifyAndGenerateToken(string token, string refreshToken)
        {
            var jwtTokenHandler = new JwtSecurityTokenHandler();

            try
            {
                // 验证JWT令牌格式
                var tokenInVerification = jwtTokenHandler.ValidateToken(token, _tokenValidationParameters, out var validatedToken);

                // 验证加密算法
                if (validatedToken is JwtSecurityToken jwtSecurityToken)
                {
                    var result = jwtSecurityToken.Header.Alg.Equals(SecurityAlgorithms.HmacSha256, StringComparison.InvariantCultureIgnoreCase);

                    if (result == false)
                    {
                        return null;
                    }
                }

                // 验证过期日期
                var utcExpityDate = long.Parse(tokenInVerification.Claims.FirstOrDefault(x => x.Type == JwtRegisteredClaimNames.Exp).Value);

                var expiryDate = utcExpityDate.UnixTimeStampToDateTime();
                if (expiryDate > DateTime.UtcNow)
                {
                    return new AuthResult
                    {
                        Success = true,
                        Errors = new List<string>() {
                            "Token has not yet expired"
                        }
                    };
                }

                // 验证令牌的存在
                var storedToken = await _myContext.RefreshTokens.FirstOrDefaultAsync(x => x.Token == refreshToken);

                if (storedToken == null)
                {
                    return new AuthResult
                    {
                        Success = false,
                        Errors = new List<string>() {
                            "Token does not exist"
                        }
                    };
                }

                // 验证是否使用
                if (storedToken.IsUsed)
                {
                    return new AuthResult
                    {
                        Success = false,
                        Errors = new List<string>() {
                            "Token has been used"
                        }
                    };
                }

                // 验证是否被撤销
                if (storedToken.IsRevorked)
                {
                    return new AuthResult
                    {
                        Success = false,
                        Errors = new List<string>() {
                            "Token has been revoked"
                        }
                    };
                }

                // 验证refreshToken是否过期
                if (storedToken.ExpiryDate < DateTime.UtcNow)
                {
                    return new AuthResult
                    {
                        Success = false,
                        Errors = new List<string>() {
                            "Token has expired please re-login"
                        }
                    };
                }

                // 核实身份
                var jti = tokenInVerification.Claims.FirstOrDefault(x => x.Type == JwtRegisteredClaimNames.Jti).Value;

                if (storedToken.JwtId != jti)
                {
                    return new AuthResult
                    {
                        Success = false,
                        Errors = new List<string>() {
                            "Token doesn't match"
                        }
                    };
                }

                // 更新当前令牌
                storedToken.IsUsed = true;
                _myContext.RefreshTokens.Update(storedToken);
                await _myContext.SaveChangesAsync();

                // 生成一个新的令牌
                var writer = await _writerInfoService.FindAsync(x => x.Id == storedToken.UserId);

                return await GenerateJwtToken(writer);
            }
            catch (Exception ex)
            {
                return ex.Message.Contains("Lifetime validation failed. The token is expired.")
                    ? new AuthResult
                    {
                        Success = false,
                        Errors = new List<string>() {
                            "Token has expired please re-login"
                        }
                    }
                    : new AuthResult()
                    {
                        Success = false,
                        Errors = new List<string>() {
                           $"{ex.Message}"
                        }
                    };
            }
        }

        private string RandomString(int length)
        {
            var random = new Random();
            var chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
            return new string(Enumerable.Repeat(chars, length)
                .Select(x => x[random.Next(x.Length)]).ToArray());
        }
    }
}