﻿using Microsoft.EntityFrameworkCore;

using MyBlog.Entity;
using MyBlog.IRepository;

using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Threading.Tasks;

namespace MyBlog.Repository
{
    public class BaseRepository<TEntity> : IBaseRepository<TEntity> where TEntity : class, new()
    {
        private MyContext _myContext;
        private DbSet<TEntity> entities;

        public BaseRepository(MyContext myContext)
        {
            _myContext = myContext;
            entities = myContext.Set<TEntity>();
        }

        public async Task<TEntity> CreateAsync(TEntity entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("entity");
            }

            try
            {
                await entities.AddAsync(entity);
                await _myContext.SaveChangesAsync();

                return entity;
            }
            catch (Exception e)
            {
                throw new Exception($"{nameof(entity)} could not be saved:{e.Message}");
            }
        }

        public virtual async Task<TEntity> DeleteAsync(TEntity entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException($"{nameof(entity)} is not found");
            }

            try
            {
                entities.Remove(entity);
                await _myContext.SaveChangesAsync();

                return entity;
            }
            catch (Exception e)
            {
                throw new Exception($"{nameof(entity)} could not be deleted: {e.Message}");
            }
        }

        public async Task<TEntity> EditAsync(TEntity entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException($"{nameof(entity)} entity must be not null");
            }

            try
            {
                entities.Update(entity);
                await _myContext.SaveChangesAsync();

                return entity;
            }
            catch (Exception e)
            {
                throw new Exception($"{nameof(entity)} could not be updated:{e.Message}");
            }
        }

        public virtual async Task<TEntity> FindAsync(int id)
        {
            return await entities.FindAsync(id);
        }

        public async Task<TEntity> FindAsync(Expression<Func<TEntity, bool>> func)
        {
            var query = entities.Where(func);
            return await query.FirstOrDefaultAsync();
        }

        public virtual async Task<List<TEntity>> QueryAsync()
        {
            return await entities.ToListAsync();
        }

        public virtual async Task<List<TEntity>> QueryAsync(Expression<Func<TEntity, bool>> func)
        {
            var query = entities.Where(func);
            return await query.ToListAsync();
        }

        public virtual async Task<List<TEntity>> QueryAsync(int page, int size)
        {
            return await entities.Skip(size * (page - 1)).Take(size).ToListAsync();
        }

        public virtual async Task<List<TEntity>> QueryAsync(Expression<Func<TEntity, bool>> func, int page, int size)
        {
            var funcInfo = entities.Where(func);

            return await funcInfo.Skip(size * (page - 1)).Take(size).ToListAsync();
        }
    }
}