﻿using AutoMapper;

using Microsoft.AspNetCore.Mvc;

using MyBlog.IService;
using MyBlog.Model;
using MyBlog.Model.DTO;
using MyBlog.Server.Utility.Filter;
using MyBlog.Shared.Extensions;
using MyBlog.Shared.Utility.ApiResult;

using System;
using System.Linq;
using System.Threading.Tasks;

namespace MyBlog.Server.Controllers
{
    [Route("api/[controller]/[action]")]
    [ApiController]
    public class BlogNewsController : ControllerBase
    {
        private readonly IBlogNewsService _blogNewsService;
        private readonly IWriterInfoService _writerInfoService;
        private readonly ITypeInfoService _typeInfoService;
        private readonly IMapper _mapper;

        public BlogNewsController(IBlogNewsService blogNewsService, IWriterInfoService writerInfoService, ITypeInfoService typeInfoService, IMapper mapper)
        {
            _blogNewsService = blogNewsService;
            _writerInfoService = writerInfoService;
            _typeInfoService = typeInfoService;
            _mapper = mapper;
        }

        [HttpGet]
        [TypeFilter(typeof(CustomResourceFilterAttribute))]
        public async Task<ActionResult<ApiResult>> GetBlogNewsByWriterId(int id)
        {
            var data = await _blogNewsService.QueryAsync(x => x.WriterId == id);
            var writer = await _writerInfoService.QueryAsync(x => x.Id == id);

            var result = data.AsEnumerable()
                 .Select(x => new BlogNewsListDTO()
                 {
                     Id = x.Id,
                     Title = x.Title,
                     Year = x.Time.Year,
                     CreateTime = x.Time.TryToDateTime()
                 })
                    .GroupBy(x => x.Year)
                    .Select(x => new BlogNewsDTO
                    {
                        Year = x.Key,
                        Name = writer[0].Name,
                        BlogNewsList = x.ToList()
                    });

            return new ApiResult(ResultCode.SUCCESS, result);
        }

        [HttpGet]
        [TypeFilter(typeof(CustomResourceFilterAttribute))]
        public async Task<ActionResult<ApiResult>> GetBlogNewsByTypeInfoId(int id)
        {
            var typeInfo = await _typeInfoService.QueryAsync(x => x.Id == id);

            var result = typeInfo.FirstOrDefault()
                .Blogs.AsEnumerable()
                 .Select(x => new BlogNewsListDTO()
                 {
                     Id = x.Id,
                     Title = x.Title,
                     Year = x.Time.Year,
                     CreateTime = x.Time.TryToDateTime()
                 })
                    .GroupBy(x => x.Year)
                    .Select(x => new BlogNewsDTO
                    {
                        Year = x.Key,
                        Name = typeInfo[0].Name,
                        BlogNewsList = x.ToList()
                    });

            return new ApiResult(ResultCode.SUCCESS, result);
        }

        [HttpGet]
        [TypeFilter(typeof(CustomResourceFilterAttribute))]
        public async Task<ActionResult<ApiResult>> GetBlogNewsDetail(int id)
        {
            var blogNews = await _blogNewsService.FindAsync(x => x.Id == id);

            if (blogNews == null)
            {
                return new ApiResult(ResultCode.FALL, "Posts without this name");
            }

            var writer = await _writerInfoService.FindAsync(x => x.Id == blogNews.WriterId);

            var entity = new BlogNewsDetailDTO
            {
                Id = blogNews.Id,
                Title = blogNews.Title,
                Content = blogNews.Content,
                HtmlContent = blogNews.HtmlContent,
                Time = blogNews.Time.TryToDateTime(),
                TypeInfoId = blogNews.TypeInfoId,
                WriterName = writer.Name,
            };

            return new ApiResult(ResultCode.SUCCESS, entity);
        }

        [HttpPost]
        public async Task<ActionResult<ApiResult>> Create(BlogNewsDetailDTO blogNewsDetail)
        {
            BlogNews blogNews = new BlogNews
            {
                Title = blogNewsDetail.Title,
                BrowseCount = 0,
                Content = blogNewsDetail.Content,
                HtmlContent = blogNewsDetail.HtmlContent,
                LikeCount = 0,
                Time = DateTime.Now,
                WriterId = Convert.ToInt32(User.FindFirst("Id").Value),
                TypeInfoId = blogNewsDetail.TypeInfoId
            };

            try
            {
                var entity = await _blogNewsService.CreateAsync(blogNews);

                return blogNews != entity ? new ApiResult(ResultCode.FALL) : (ActionResult<ApiResult>)new ApiResult(ResultCode.SUCCESS, blogNews);
            }
            catch (Exception e)
            {
                return new ApiResult(ResultCode.FALL, $"{e.Message}");
            }
        }

        [HttpDelete]
        public async Task<ActionResult<ApiResult>> Delete(int id)
        {
            var blogNews = await _blogNewsService.FindAsync(id);

            try
            {
                var entity = await _blogNewsService.DeleteAsync(blogNews);

                return blogNews != entity ? new ApiResult(ResultCode.FALL, "删除失败") : (ActionResult<ApiResult>)new ApiResult(ResultCode.SUCCESS, blogNews);
            }
            catch (Exception e)
            {
                return new ApiResult(ResultCode.FALL, $"{e.Message}");
            }
        }

        [HttpPut]
        public async Task<ActionResult<ApiResult>> Edit(BlogNewsDetailDTO blogNewsDetail)
        {
            var blogNews = await _blogNewsService.FindAsync(blogNewsDetail.Id);

            blogNews.Title = blogNewsDetail.Title;
            blogNews.Content = blogNewsDetail.Content;
            blogNews.HtmlContent = blogNewsDetail.HtmlContent;
            blogNews.TypeInfoId = blogNewsDetail.TypeInfoId;
            blogNews.Time = DateTime.Now;

            try
            {
                var entity = await _blogNewsService.EditAsync(blogNews);

                return blogNews != entity ? new ApiResult(ResultCode.FALL, "修改文章失败") : new ApiResult(ResultCode.SUCCESS, blogNews);
            }
            catch (Exception e)
            {
                return new ApiResult(ResultCode.FALL, $"{e.Message}");
            }
        }

        [HttpGet]
        [TypeFilter(typeof(CustomResourceFilterAttribute))]
        public async Task<ApiResult> GetBlogNewsPage(int page, int size)
        {
            var entity = await _blogNewsService.QueryAsync(page, size);

            try
            {
                var blogNews = entity.AsEnumerable()
                    .Select(x => new BlogNewsListDTO()
                    {
                        Id = x.Id,
                        Title = x.Title,
                        Year = x.Time.Year,
                        CreateTime = x.Time.TryToDateTime()
                    })
                    .GroupBy(x => x.Year)
                    .Select(x => new BlogNewsDTO
                    {
                        Year = x.Key,
                        BlogNewsList = x.ToList()
                    });

                return new ApiResult(ResultCode.SUCCESS, blogNews);
            }
            catch (Exception e)
            {
                return new ApiResult(ResultCode.FALL, $"{e.Message}");
            }
        }
    }
}