﻿using AutoMapper;

using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;

using MyBlog.IService;
using MyBlog.Model;
using MyBlog.Model.DTO;
using MyBlog.Server.Utility.Filter;
using MyBlog.Shared.Utility.ApiResult;
using MyBlog.Shared.Utility.MD5Utility;

using System;
using System.Linq;
using System.Threading.Tasks;

namespace MyBlog.Server.Controllers
{
    [Route("api/[controller]/[action]")]
    [ApiController]
    public class WriterInfoController : ControllerBase
    {
        private readonly IWriterInfoService _writerInfoService;
        private readonly IBlogNewsService _blogNewsService;
        private readonly IMapper _mapper;

        public WriterInfoController(IWriterInfoService writerInfoService, IBlogNewsService blogNewsService, IMapper mapper)
        {
            _writerInfoService = writerInfoService;
            _blogNewsService = blogNewsService;
            _mapper = mapper;
        }

        [HttpPost]
        [Authorize]
        public async Task<ApiResult> Create(WriterInfo writer)
        {
            WriterInfo writerInfo = new WriterInfo
            {
                Name = writer.Name,
                UserPwd = MD5Helper.MD5Encrypt32(writer.UserPwd),
                UserName = writer.UserName
            };

            var isExits = await _writerInfoService.FindAsync(x => x.UserName == writer.UserName);
            if (isExits != null)
            {
                return new ApiResult(ResultCode.FALL, "Account already exists");
            }

            try
            {
                var entity = await _writerInfoService.CreateAsync(writerInfo);
                return entity != writerInfo ? new ApiResult(ResultCode.FALL, "Failed to add") : new ApiResult(ResultCode.SUCCESS, entity);
            }
            catch (Exception e)
            {
                return new ApiResult(ResultCode.FALL, $"{e.Message}");
            }
        }

        [HttpDelete]
        [Authorize]
        public async Task<ApiResult> Delete(int id)
        {
            var writer = await _writerInfoService.FindAsync(id);

            if (writer == null)
            {
                return new ApiResult(ResultCode.FALL);
            }

            try
            {
                var entity = await _writerInfoService.DeleteAsync(writer);
                return entity != writer ? new ApiResult(ResultCode.FALL, "Delete Fall") : new ApiResult(ResultCode.SUCCESS, entity);
            }
            catch (Exception e)
            {
                return new ApiResult(ResultCode.FALL, $"{e.Message}");
            }
        }

        [HttpPut]
        [Authorize]
        public async Task<ApiResult> Edit(WriterInfo writer)
        {
            int id = Convert.ToInt32(User.FindFirst("Id").Value);

            var writerInfo = await _writerInfoService.FindAsync(id);

            writerInfo.Name = writer.Name;
            writerInfo.UserName = writer.UserName;
            writerInfo.UserPwd = writer.UserPwd;

            try
            {
                var entity = await _writerInfoService.EditAsync(writerInfo);
                return entity != writerInfo ? new ApiResult(ResultCode.FALL, "Edit Fall") : new ApiResult(ResultCode.SUCCESS, entity);
            }
            catch (Exception e)
            {
                return new ApiResult(ResultCode.FALL, $"{e.Message}");
            }
        }

        [HttpGet]
        [TypeFilter(typeof(CustomResourceFilterAttribute))]
        public async Task<ApiResult> FindWriter(int id)
        {
            var writer = await _writerInfoService.FindAsync(id);

            if (writer == null)
            {
                return new ApiResult(ResultCode.FALL, $"Id {id} not found");
            }

            var entity = _mapper.Map<WriterInfoDTO>(writer);
            return new ApiResult(ResultCode.SUCCESS, entity);
        }

        [HttpGet]
        [TypeFilter(typeof(CustomResourceFilterAttribute))]
        public async Task<ApiResult> GetAllWriters()
        {
            var writers = await _writerInfoService.QueryAsync();
            var blogNews = await _blogNewsService.QueryAsync();

            var entity = writers.Select(x => new WriterInfoDTO
            {
                Id = x.Id,
                Name = x.Name,
                UserName = x.UserName,
                Count = blogNews.Count(p => p.WriterId == x.Id)
            });

            return new ApiResult(ResultCode.SUCCESS, entity);
        }
    }
}